﻿package aibase
{
	internal class CollisionAvoidance implements GroupStrategy
	{
		public static  const INSTANCE:GroupStrategy = new CollisionAvoidance(Globals.AVOID_RADIUS()
		        ,Globals.AVOID_TIME());
		public function getName():String
		{
			return "CollisionAvoidance";
		}
		private var radius:Number;
		private var maxTime:Number;
		private var neighbors:Array;

		public function CollisionAvoidance(_radius:Number, _maxTime:Number)
		{
			radius = _radius;
			maxTime = _maxTime;
		}
		public function setNeighbors(_others:Array):void
		{
			neighbors = _others;
		}
		public function getSteering(a:Bot, b:Bot):SteeringOutput
		{

			var firstTarget:Bot = null;
			var shortestTime:Number = 99999;
			var firstMinSeperation:Number = 0;
		    var firstDistance:Number = 0;
			var firstRelativePos:Vec2f = new Vec2f(0,0);
			var firstRelativeVel = new Vec2f(0,0);

			for (var x:int=0; x<neighbors.length; x++)
			{
				b = neighbors[x];
				if (b == a)
				{
					b = BotArena.Hero;//don't include yourself, but do include hero
				}
				var relativePos:Vec2f = Utils.difference(a.kinematic.position,b.kinematic.position);

				var relativeVel:Vec2f = Utils.sub(a.kinematic.velocity, b.kinematic.velocity);
				var relativeSpeed:Number = Utils.l2(relativeVel);
				var timeToCollision:Number = Utils.dot(relativePos,relativeVel) / (relativeSpeed * relativeSpeed);
				var distance:Number = Utils.l2(relativePos);
				var minSeperation:Number = distance - relativeSpeed*timeToCollision;
				if (minSeperation > 2*radius)
				{
					continue;
				}//no collision...

				//otherwise is it the worst?
				if (timeToCollision > 0 && timeToCollision < shortestTime)
				{
					shortestTime = timeToCollision;
					firstTarget = b;
					firstMinSeperation = minSeperation;
					firstDistance = distance;
					firstRelativePos = relativePos;
					firstRelativeVel = relativeVel;
				}
			}


			if (firstTarget != null && shortestTime < maxTime)
			{
				//var relativePos:Vec2f;
				if (firstMinSeperation <= 0 || firstDistance < 2*radius)
				{
					relativePos = Utils.difference(a.kinematic.position,firstTarget.kinematic.position);
				}
				else
				{
					relativePos = Utils.add(firstRelativePos,Utils.mult(firstRelativeVel,shortestTime));
				}

				relativePos = Utils.normalize(relativePos);
				var location:Vec2f = Utils.add(a.kinematic.position,relativePos);
				var fake:Bot = new Bot();
				fake.kinematic.position = location;
				return DynamicFlee.INSTANCE.getSteering(a,fake);
			}
			else
			{
				return new SteeringOutput(new Vec2f(0,0),0,0);
			}
		}
	}
}