﻿package aibase
{
	class Utils
	{
		static var WIDTH:Number = 640;
		static var HEIGHT:Number = 480;
		static const PAD:Number = 5;
		static const RADIANS_TO_DEGREES:Number = 57.2958;

		
		//returns Vec2f(-99999,-99999) if no cross
		static function LinesCross( Ax:Number, Ay:Number, Bx:Number, By:Number,
		   Cx:Number, Cy:Number, Dx:Number, Dy:Number):Vec2f
		{
			http://cgafaq.info/wiki/Intersection_Of_2D_Line_Segments
			var denom:Number= (Bx-Ax)*(Dy-Cy)-(By-Ay)*(Dx-Cx)
			var rTop:Number = (Ay-Cy)*(Dx-Cx)-(Ax-Cx)*(Dy-Cy);
			var sTop:Number = (Ay-Cy)*(Bx-Ax)-(Ax-Cx)*(By-Ay)
			if(denom == 0 || rTop == 0)
			{
				return new Vec2f(-99999,-99999);
			}
			var r:Number = rTop/denom;
			var s:Number = sTop/denom;
			if(r > 0 && r < 1)
			{
				var px:Number = Ax + r*(Bx-Ax);
				var py:Number = Ay + r*(By-Ay);
				return new Vec2f(px,py);
			}
			else
			{
				return new Vec2f(-99999,-99999);
			}				
		}
		
		
		//ripped from http://www.gamedev.net/community/forums/topic.asp?topic_id=380428
		/*{
			var d:Number=(x1-x0)*(y3-y2)-(y1-y0)*(x3-x2);
			if (Math.abs(d)<0.001)
			{
				return new Vec2f(-99999,-99999);
			}
			var AB:Number =((y0-y2)*(x3-x2)-(x0-x2)*(y3-y2))/d;
			if (AB>0.0 && AB<1.0)
			{
				var CD:Number =((y0-y2)*(x1-x0)-(x0-x2)*(y1-y0))/d;
				if (CD>0.0 && CD<1.0)
				{
					var linx:Number =x0+AB*(x1-x0);
					var liny:Number =y0+AB*(y1-y0);
					return new Vec2f(linx,liny);
				}
			}
			
			return new Vec2f(-99999,-99999);
    	}*/

		static function rotate(p_vec:Vec2f, rot:Number):Vec2f
		{
			return new Vec2f(p_vec.x * Math.sin(rot),p_vec.y * Math.cos(rot));
		}
		static function add(a:Vec2f, b:Vec2f):Vec2f
		{
			return new Vec2f(a.x + b.x,a.y + b.y);
		}
		static function sub(a:Vec2f, b:Vec2f):Vec2f
		{
			return new Vec2f(a.x - b.x,a.y - b.y);
		}
		static function mult(a:Vec2f, scalar:Number):Vec2f
		{
			return new Vec2f(a.x * scalar,a.y * scalar);
		}
		
		static function peiceWise(a:Vec2f, b:Vec2f):Vec2f
		{
			return new Vec2f(a.x * b.x , a.y * b.y);
		}
		
		static function orientationFromDirection(p_vec:Vec2f):Number
		{
			p_vec = Utils.normalize(p_vec);
			return Math.atan2(p_vec.y,p_vec.x);//woah...
		}
		static function directionFromOrientation(orient:Number):Vec2f
		{
			return new Vec2f(Math.cos(orient),Math.sin(orient));
		}
		static function l2(a:Vec2f):Number
		{
			return Math.sqrt(a.x * a.x + a.y * a.y);
		}
		static function normalize(p_vec:Vec2f):Vec2f
		{
			var len = l2(p_vec);
			if (len == 0)
			{
				return new Vec2f(p_vec.x,p_vec.y);
			}
			else
			{
				return mult(p_vec,1 / len);
			}
		}
		static function scaleToLength(a:Vec2f , p_length:Number):Vec2f
		{
			var r = normalize(a);
			return mult(r,p_length);
		}
		//these should be in torus,b ut i'll make sure.
		static function difference(a:Vec2f, b:Vec2f):Vec2f
		{

			if (!Globals.TORUS())
			{
				return Utils.sub(b,a);
			}
			a = positionToTorus(a);
			b = positionToTorus(b);
			//calculate x and y seperate.


			var resultX;
			var resultY;

			///X    
			//normal route.
			var forward = Math.abs(a.x-b.x);

			//now figure out distance for wrapping:
			var l = Math.min(a.x,b.x);
			var r = Math.max(a.x,b.x);
			var wrap = l + WIDTH-r;
			if (wrap < forward)
			{
				resultX = -wrap;

			}
			else
			{
				resultX = forward;
			}
			if (a.x > b.x)
			{
				resultX *= -1;//we went left to get there
			}
			// Y
			var t = Math.min(a.y,b.y);
			var bottom = Math.max(a.y,b.y);
			forward = Math.abs(a.y-b.y);
			wrap = t + HEIGHT-bottom;
			if (wrap < forward)
			{
				resultY = -wrap;
			}
			else
			{
				resultY = forward;
			}
			if (a.y > b.y)
			{
				resultY *= -1;
			}

			return new Vec2f(resultX,resultY);
		}

		static function positionToTorus(a:Vec2f):Vec2f
		{
			if (!Globals.TORUS())
			{
				return new Vec2f(clamp(a.x,PAD,WIDTH-PAD),clamp(a.y,PAD,HEIGHT-PAD));
			}
			var tempx = a.x;
			var tempy = a.y;
			while (true)
			{
				var delta = false;
				if (tempx < 0)
				{
					tempx += WIDTH;
					delta =true;
				}
				else if (tempx >= WIDTH)
				{
					tempx -= WIDTH;
					delta = true;
				}
				if (tempy < 0)
				{
					tempy += HEIGHT;
					delta = true;
				}
				else if (tempy >= HEIGHT)
				{
					tempy -= HEIGHT;
					delta = true;
				}
				if (!delta)
				{
					return new Vec2f(tempx,tempy);
				}
			}
			return Vec2f.zero();//you better never get here

		}
		static function mapToRange(val:Number, low:Number, high:Number, delta:Number):Number
		{
			while (val < low)
			{
				val += delta;
			}
			while (val > high)
			{
				val -= delta;
			}
			return val;
		}

		static function clamp(val:Number, low:Number, high:Number):Number
		{
			if (val < low)
			{
				val = low;
			}
			if (val > high)
			{
				val = high;
			}
			return val;
		}
		static function dot(a:Vec2f, b:Vec2f):Number
		{
			return a.x * b.x + a.y * b.y;
		}
		
		// Method to calculate the length of a Vec2f 
		static function Vec2fLength(a : Vec2f) : Number{
			return Math.sqrt(a.x * a.x + a.y * a.y);
		}
	}
}